#!/bin/bash

###
# Deploys and orchestrate all VM Based Operations, commanded by the WebService.
# 
#	- Makes sure we can track what Host/Stack a given VM is running onto.
#	
#	HOST --> [Host Runner] --> STACK (Hypervisor) --> Node (VM) --> [Node Runner]
#	>> There is no way to guess what host we're onto with a given VM.
#		- Option 1: Use guest properties on all VMs to get a HostID : Rejected cause we need to parse constantly the result and thats combersome. Also installing the tools on every single machine (err windows)
#		- Option 2: Use a local rest service on a "Host Domain Controller" so we can have the "Host UUID" permitting to locate where the Node is running. [Selected]
###
#	/register :
#		- register the [Host runner] to the Webservice (returns a UUID) that will be fed to the guestParameters temporarelly.
#		- [Domain Controller] : Once the temporary key has been assigned, the DC will diplay this UUID onto its webservice (http://domaincontroller.local/ws/wherearewe)
#		- If DC restarts, the external prop will disapear leaving the key only accessible by VMs from the stack.

#####

scriptPath=$(pwd)
responseFilePath=$scriptPath'/hr/responseFile.json'
#hostRunnerUID_FILE=$scriptPath'/hr/hostRunner.id'
#apiKey_FILE=$scriptPath'/store/api.key'


function mkdirParents() {
	mkdir -p $scriptPath'/hr/'
	mkdir -p $scriptPath'/store/'
}

function colorInit() {
	BLK='\033[0;30m'
	RED='\033[0;31m'
	GRN='\033[0;32m'
	ORA='\033[0;33m'
	BLU='\033[0;34m'
	PUR='\033[0;35m'
	CYA='\033[0;36m'
	LGR='\033[0;37m'
	DGR='\033[1;30m'
	LRD='\033[1;31m'
	LGN='\033[1;32m'
	YEL='\033[1;33m'
	LBL='\033[1;34m'
	LPP='\033[1;35m'
	LCY='\033[1;36m'
	WHT='\033[1;37m'
	NC='\033[0m'
}

function retrieveContentIfFileSetup() {
	# Check if we've an API Key Setup? (~/iq_tools/store/api.key) <> use, it | request user
	fileSetup=$(echo $1)

	if test -f "$fileSetup"; then
		ret=$(cat $fileSetup)
	else
		ret=''
	fi

	echo "$ret"
}

function registerHost() {
	echo 'lol'
}

function getOSTicker() {
	LSB_DID=$(lsb_release -a 2> /dev/null | grep -i 'distributor id:' | awk '{print $3}')
	LSB_RELEASE=$(lsb_release -a 2> /dev/null | grep -i 'release:' | awk '{print $2}')

	echo ${LSB_DID:0:3}${LSB_RELEASE}
}

function ihm_fillInformations() {
	echo -e $LGN"Your stack looks not configured, we need several information before we can go further..."$NC
	echo -en $WHT"Stack Name:"$NC; read -p " " _stackname
	echo -en $WHT"Site "$LGR"(Delta, Laptop, <TRG>):"$NC;read -p " " _site
	echo -en $WHT"Owner: "$NC;read -p " " _owner
	echo -en $WHT"Host Type (mutu, perso):"$NC;read -p " " _htype
	echo -en $WHT"OS release "$LGR"(win10, deb10, osx) "$LGN"['"$(getOSTicker)"']:"$NC;read -p " " _os
	echo -en $WHT"SapiKey:"$NC;read -p " " _apiKey

	if [[ $_os == '' ]]; then _os=$(getOSTicker); fi

	echo -e " ╭―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――╮"
	echo -e " │ 			HOST Configuration			 │"
	echo -e " │ 								 │"
	echo -e " │ Stack Name : "$_stackname"\t\t\t\t\t │"
	echo -e " │ Site.      : "$_site"\t\t\t\t\t\t │"
	echo -e " │ Owner      : "$_owner"\t\t\t\t\t\t │"
	echo -e " │ Host Type  : "$_htype"\t\t\t\t\t\t │"
	echo -e " │ OS.        : "$_os"\t\t\t\t\t\t │"
	echo -e " │ api.       : "$_apiKey"\t │"
	echo -e " ╰―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――╯"

	echo '{ "stackName": "stackname", "site": "site", "owner": "owner", "hostType": "hosttype", "os": "os", "apiKey": "apikey"}' | jq \
	 --arg STACKNAME "$_stackname" \
	 --arg SITE "$_site" \
	 --arg OWNER "$_owner" \
	 --arg HTYPE "$_htype" \
	 --arg OS "$_os" \
	 --arg APIK "$_apiKey" \
	 '.stackName |= $STACKNAME | .site |= $SITE | .owner |= $OWNER | .hostType |= $HTYPE | .os |= $OS | .apiKey |= $APIK' > $responseFilePath
}

colorInit

# stack_key-VLU-284c91c56da11ca04b79c6f4f1654f0a
# generic-stack-nr-vlu-effc6bde58d799ac1711be910dc6e209
# 
# generic-web-auth-token
# generic-stack-singleuse-token
# user-querry-apikey

# generic-stack-hostrunner-token
# generic-stack-noderunner-token

#if [[ "$STR" == *"$SUB"* ]]; then
#	#statements
#fi

# STEP 1 : Am I registered ?
# --> ~/iq_tools/hr/hostRunner.id contains an ID ?
__RFILE_CONTENT=$(retrieveContentIfFileSetup $responseFilePath);

echo 'RespFilepath: '$responseFilePath
echo 'Contents: '$__RFILE_CONTENT

if [[ $__RFILE_CONTENT == '' ]]; then
	# Not present --> Not registred. need to register
	ihm_fillInformations


else
	# Probably registered
	# Get API Key

	# Validate UID
	echo "bulshit"


fi



### STEPS
# STEP 1 : Am I registered ?
# --> ~/iq_tools/hr/hostRunner.id contains an ID ?

	# NO I AM NOT 
	# STEP 1.1 : Check if we have an answer file (Name, Site, Owner, TMP API Key, ...)
	# STEP 1.2 : Check dependencies
	# STEP 1.3 : Start Register Host
	# STEP 1.4 : Create Stack
	# STEP 1.5 : Bind Stack --> Host
	# STEP 1.6 : Deploy DC
	# STEP 1.7 : Pass Guest Prop
	# STEP 1.8 : Fetch DC REG Status
	# STEP 1.9 : Bind DC --> Stack



### Next secu design ?
# We can try to obfuscate the API (not storing it in local)
# --> calling service API to retrieve a token at a given time ?
# --> then we need to authenticate the call
# --> callback can then be (UID)*PUB_IP ?

### Check VBOXManager
# VBoxManage guestproperty set mini-deb-bkb-netbalancer QA_AUTOMATION_DOMAIN_CONTROL_TOKEN ajvxx-45ws4-u27s887 
# set <vm> <property> [<value> [--flags <flags>]]: Enables you to set a guest property by specifying the keyword and value. If <value> is omitted, the property is deleted. With --flags, you can specify additional behavior. You can combine several flags by separating them with commas.
#	TRANSIENT: The value will not be stored with the VM data when the VM exits.
#	TRANSRESET: The value will be deleted as soon as the VM restarts or exits.
#	RDONLYGUEST: The value can only be changed by the host, but the guest can only read it.
#	RDONLYHOST: The value can only be changed by the guest, but the host can only read it.
#	READONLY: The value cannot be changed at all.
#
### Install guest-addition on DC.
# https://linuxize.com/post/how-to-install-virtualbox-guest-additions-on-debian-10/
#
# ← ↑ → ↓ ↔ ↕ ↖ ↗ ↘ ↙ ↚ ↛ ↜ ↝ ↞ ↟ ↠ ↡ ↢ ↣ ↤ ↥ ↦ ↧ ↨ ↩ ↪ ↫ ↬ ↭ ↮ ↯ ↰ ↱ ↲ ↳ ↴ ↵ ↶ ↷ ↸ ↹ ↺ ↻ ⇄ ⇅ ⇆ ⇇ ⇈ ⇉ 
#
# ╭――――――――――――――――――╮
# │                  │
# │                  │
# │                  │
# ╰――――――――――――――――――╯